/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.ow2.bonita.async.AsyncTest;
import org.ow2.bonita.env.XpdlEnvironmentParserTest;
import org.ow2.bonita.env.descriptor.RuntimeDbSessionDescriptorTest;
import org.ow2.bonita.env.generator.DbHistoryEnvGenerator;
import org.ow2.bonita.env.generator.XmlHistoryEnvGenerator;
import org.ow2.bonita.facade.InternalAPITest;
import org.ow2.bonita.services.impl.ChainFinishedInstanceHandlerTest;
import org.ow2.bonita.services.impl.ChainUndeployedPackageHandlerTest;
import org.ow2.bonita.services.impl.CleanJournalFinishedInstanceHandlerTest;
import org.ow2.bonita.services.impl.CleanJournalUndeployedPackageHandlerTest;
import org.ow2.bonita.services.impl.NoOpUndeployedPackageHandlerTest;
import org.ow2.bonita.services.record.DbTaskInstTest;
import org.ow2.bonita.services.record.HistoryTest;
import org.ow2.bonita.services.record.JournalTest;
import org.ow2.bonita.util.EnvToolTest;
import org.ow2.bonita.util.ToolTest;

/**
 * @author "Pierre Vigneras"
 * @date Dec 5, 2007
 */
public final class NonRegressionTests extends TestCase {
  private NonRegressionTests() { }

  public static Test suite() {
    final TestSuite suite = new TestSuite(NonRegressionTests.class.getName());

    // Tests that use the default envGenerator
    final TestSuite defaultEnvSuite = new TestSuite();
    defaultEnvSuite.addTest(DefaultEnvNonRegressionTests.suite());
    defaultEnvSuite.addTestSuite(JournalTest.class);
    defaultEnvSuite.addTestSuite(HistoryTest.class);
    defaultEnvSuite.addTestSuite(ToolTest.class);
    defaultEnvSuite.addTestSuite(EnvToolTest.class);
    defaultEnvSuite.addTestSuite(InternalAPITest.class);
    defaultEnvSuite.addTestSuite(AsyncTest.class);
    suite.addTest(new EnvironmentFactoryTestSetup(defaultEnvSuite, new DbHistoryEnvGenerator()));
    // tests with XmlHistory environment
    suite.addTest(new EnvironmentFactoryTestSetup(ProxiedTests.suite(defaultEnvSuite),
        new XmlHistoryEnvGenerator()));

    // TESTS that DO NOT use the default EnvGenerator
    // These tests are not launched in EJB tests
    suite.addTestSuite(RuntimeDbSessionDescriptorTest.class);
    suite.addTestSuite(DbTaskInstTest.class);
    suite.addTestSuite(XpdlEnvironmentParserTest.class);

    // TODO: fix these tests to make them executable from eclipse.
    suite.addTest(new EnvironmentFactoryTestSetup(new TestSuite(CleanJournalFinishedInstanceHandlerTest.class),
        CleanJournalFinishedInstanceHandlerTest.getEnvGenerator()));
    suite.addTest(new EnvironmentFactoryTestSetup(new TestSuite(CleanJournalUndeployedPackageHandlerTest.class),
        CleanJournalUndeployedPackageHandlerTest.getEnvGenerator()));
    suite.addTest(new EnvironmentFactoryTestSetup(new TestSuite(NoOpUndeployedPackageHandlerTest.class),
        NoOpUndeployedPackageHandlerTest.getEnvGenerator()));
    suite.addTest(new EnvironmentFactoryTestSetup(new TestSuite(ChainFinishedInstanceHandlerTest.class),
        ChainFinishedInstanceHandlerTest.getEnvGenerator()));
    suite.addTest(new EnvironmentFactoryTestSetup(new TestSuite(ChainUndeployedPackageHandlerTest.class),
        ChainUndeployedPackageHandlerTest.getEnvGenerator()));

    return suite;
  }
}
