package org.ow2.bonita.activity.route;

import java.net.URL;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.activity.instantiation.instantiator.TestOneJoinActivityInstantiator;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.BonitaException;


public class CancelJoinXorTest extends APITestCase {

  public void testPartialJoinMultiInstantiation() throws BonitaException {
    final URL xpdlUrl = this.getClass().getResource("joinXorCancel.xpdl");
    final String processId = "joinXorCancel";

    final Map<String, ProcessDefinition> processMap = getManagementAPI().deployBar(getBarContent(xpdlUrl,
      TestOneJoinActivityInstantiator.class));
    final ProcessDefinition process = processMap.get(processId);
    final PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    final ProcessDefinitionUUID processUUID = process.getUUID();
    final ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    final Set<ActivityInstance<ActivityBody>> actIntances = getQueryRuntimeAPI().getActivityInstances(instanceUUID, "r1");
    assertEquals(1, actIntances.size());


    for (final ActivityInstance<ActivityBody> activityInstance : actIntances) {
      final ActivityBody body = activityInstance.getBody();
      if (body instanceof TaskInstance) {
        final TaskUUID taskUUID = ((TaskInstance) body).getUUID();
        getRuntimeAPI().startTask(taskUUID, false);
        getRuntimeAPI().finishTask(taskUUID, false);
      }
    }
    Collection<ActivityInstance<TaskInstance>> tasks = getQueryRuntimeAPI().getTaskList(instanceUUID, ActivityState.READY);
    assertEquals(0, tasks.size());
    tasks = getQueryRuntimeAPI().getTaskList(instanceUUID, ActivityState.ABORTED);
    assertEquals(1, tasks.size());

    checkExecutedOnce(instanceUUID, new String[]{"r1", "BonitaEnd"});

    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testRouteWithOnlyXORJoin() throws BonitaException {
    final URL xpdlUrl = this.getClass().getResource("joinXorCancelAutomaticActivities.xpdl");
    final String processId = "joinXorCancelTree";

    final Map<String, ProcessDefinition> processMap = getManagementAPI().deployBar(getBarContent(xpdlUrl));
    final ProcessDefinition process = processMap.get(processId);
    final PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    final ProcessDefinitionUUID processUUID = process.getUUID();
    final ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    final Set<ActivityInstance<ActivityBody>> actIntances = getQueryRuntimeAPI().getActivityInstances(instanceUUID);
    // shortest path to end is 6 activities
    // check shortest path is taken
    assertEquals(6, actIntances.size());

    checkOnlyOneExecuted(instanceUUID, new String[]{"a", "b"});
    checkOnlyOneExecuted(instanceUUID, new String[]{"a1", "b1", "c"});
    checkOnlyOneExecuted(instanceUUID, new String[]{"d", "e"});
    checkExecutedOnce(instanceUUID, new String[]{"f", "BonitaEnd"});

    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testRouteWithXORandANDJoin() throws BonitaException {
    final URL xpdlUrl = this.getClass().getResource("joinXorAndCancelAutomaticActivities.xpdl");
    final String processId = "joinXorCancelTree";

    final Map<String, ProcessDefinition> processMap = getManagementAPI().deployBar(getBarContent(xpdlUrl));
    final ProcessDefinition process = processMap.get(processId);
    final PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    final ProcessDefinitionUUID processUUID = process.getUUID();
    final ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    final Set<ActivityInstance<ActivityBody>> actIntances = getQueryRuntimeAPI().getActivityInstances(instanceUUID);
    // shortest path to end is 8 activities
    // f depends on d and e.
    // d depends on a and c, e on b and c
    // c depends on a XOR b
    // as a result, f depends on a AND b
    // this test simply checks b is not aborted when a arrives on c
    assertEquals(10, actIntances.size());

    checkExecutedOnce(instanceUUID, new String[]{"a", "a1", "b", "b1", "c", "d", "e", "f", "BonitaEnd"});

    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }
}
