package org.ow2.bonita.facade.businessArchive;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.element.Resource;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BusinessArchiveFactory;
import org.ow2.bonita.util.ClassDataTool;
import org.ow2.bonita.util.Misc;

public class BusinessArchiveTest extends APITestCase {

  private static final String PROCESS_ID = "businessArchive";

  public void testBusinessArchive() throws BonitaException, IOException {
    // xpdl, class, properties, xml, jar

    final String xpdl = "businessArchive.xpdl";
    final byte[] xpdlContent = Misc.getAllContentFrom(this.getClass().getResource(xpdl));
    final String jar1 = "jar1.jar";
    final byte[] jar1Content = Misc.generateJar(MyClass1.class, MyClass2.class);
    final String jar2 = "dir1/jar2.jar";
    final byte[] jar2Content = Misc.generateJar(MyClass2.class, MyClass3.class);
    final String class1 = MyClass1.class.getName() + ".class";
    final byte[] class1Content = ClassDataTool.getClassData(MyClass1.class);
    final String class2 = MyClass2.class.getName() + ".class";
    final byte[] class2Content = ClassDataTool.getClassData(MyClass2.class);
    final String form1 = "form1.xml";
    final byte[] form1Content = Misc.getAllContentFrom(this.getClass().getResource("form1.xml"));
    final String form2 = "dir1/form2.xml";
    final byte[] form2Content = Misc.getAllContentFrom(this.getClass().getResource("form2.xml"));
    final String properties = "myProps.properties";
    final byte[] propertiesContent = Misc.getAllContentFrom(this.getClass().getResource("myProps.properties"));
    
    Map<String, byte[]> resources = new HashMap<String, byte[]>();
    resources.put(xpdl, xpdlContent);
    resources.put(jar1, jar1Content);
    resources.put(jar2, jar2Content);
    resources.put(class1, class1Content);
    resources.put(class2, class2Content);
    resources.put(form1, form1Content);
    resources.put(form2, form2Content);
    resources.put(properties, propertiesContent);

    byte[] barContent = BusinessArchiveFactory.getBusinessArchiveContent(resources);
    ProcessDefinition process = getManagementAPI().deployBar(barContent).get(PROCESS_ID);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    PackageDefinition packageDef = getQueryDefinitionAPI().getPackage(packageUUID);

    BusinessArchive businessArchive = packageDef.getBusinessArchive();
    Iterator<Resource> it = null;
    
    Resource baXpdlFile = businessArchive.getXpdlFile();
    assertNotNull(baXpdlFile);
    
    Collection<Resource> baClasses = businessArchive.getClasses();
    assertNotNull(baClasses);
    assertEquals(2, baClasses.size());
    it = baClasses.iterator();
    while (it.hasNext()) {
      Resource r = it.next();
      if (r.getPath().equals(class1)) {
        Arrays.equals(class1Content, r.getData());
      } else if (r.getPath().equals(class2)) {
        Arrays.equals(class2Content, r.getData());
      } else {
        fail("Unexpected class path: " + r.getPath());
      }
    }
    
    
    Collection<Resource> baJarFiles = businessArchive.getJarFiles();
    assertNotNull(baJarFiles);
    assertEquals(2, baJarFiles.size());
    it = baJarFiles.iterator();
    while (it.hasNext()) {
      Resource r = it.next();
      if (r.getPath().equals(jar1)) {
        Arrays.equals(jar1Content, r.getData());
      } else if (r.getPath().equals(jar2)) {
        Arrays.equals(jar2Content, r.getData());
      } else {
        fail("Unexpected jar path: " + r.getPath());
      }
    }
    
    Collection<Resource> baResources = businessArchive.getResources();
    assertNotNull(baResources);
    assertEquals(resources.size(), baResources.size());
    it = baResources.iterator();
    Collection<String> baResourcesPath = new ArrayList<String>();
    while (it.hasNext()) {
      baResourcesPath.add(it.next().getPath());
    }
    assertTrue("baResources does not contain:" + xpdl, baResourcesPath.contains(xpdl));
    assertTrue("baResources does not contain:" + jar1, baResourcesPath.contains(jar1));
    assertTrue("baResources does not contain:" + jar2, baResourcesPath.contains(jar2));
    assertTrue("baResources does not contain:" + class1, baResourcesPath.contains(class1));
    assertTrue("baResources does not contain:" + class2, baResourcesPath.contains(class2));
    assertTrue("baResources does not contain:" + form1, baResourcesPath.contains(form1));
    assertTrue("baResources does not contain:" + form2, baResourcesPath.contains(form2));
    assertTrue("baResources does not contain:" + properties, baResourcesPath.contains(properties));
        
    Resource baForm2 = businessArchive.getResource(form2);
    assertNotNull(baForm2);
    Arrays.equals(form2Content, baForm2.getData());
    
    Collection<Resource> baXmlFiles = businessArchive.getResources(".*\\.xml");
    assertNotNull(baXmlFiles);
    assertEquals(2, baXmlFiles.size());
    it = baXmlFiles.iterator();
    while (it.hasNext()) {
      Resource r = it.next();
      if (r.getPath().equals(form1)) {
        Arrays.equals(form1Content, r.getData());
      } else if (r.getPath().equals(form2)) {
        Arrays.equals(form2Content, r.getData());
      } else {
        fail("Unexpected xml path: " + r.getPath());
      }
    }
    
    Collection<Resource> baDir1Files = businessArchive.getResources("^dir1/.*");
    assertNotNull(baDir1Files);
    assertEquals(2, baDir1Files.size());
    it = baDir1Files.iterator();
    while (it.hasNext()) {
      Resource r = it.next();
      if (r.getPath().equals(jar2)) {
        Arrays.equals(jar2Content, r.getData());
      } else if (r.getPath().equals(form2)) {
        Arrays.equals(form2Content, r.getData());
      } else {
        fail("Unexpected dir1/ path: " + r.getPath());
      }
    }
    
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testBusinessArchiveFactory() {

  }

  private static class MyClass1 { }
  private static class MyClass2 { }
  private static class MyClass3 { }
  
}
