package org.ow2.bonita.facade.imposterizer;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

import org.jmock.api.Invocation;
import org.jmock.api.Invokable;

public class ImpostInterceptor implements InvocationHandler {

  private Invokable mockObject;
  private Class< ? > clazz;
  private Class < ? >[] classes;
  
  public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
    if (isAnImpostClass(classes, clazz)) {
      if (method.getName().equals("hashCode")) {
        return mockUUIDHashCode(clazz, proxy);
      } else if (method.getName().equals("equals")) {
        if (args[0] != null) {
          return mockUUIDEquals(clazz, proxy, args[0]);
        }
      }
    }
    return mockObject.invoke(new Invocation(proxy, method, args));
  }
  
  private int mockUUIDHashCode(Class< ? > clazz, Object proxy) throws Throwable {
    Method m = clazz.getMethod("getUUID");
    Object o = mockObject.invoke(new Invocation(proxy,m));
    int i = o.hashCode();
    return i;
  }

  private boolean mockUUIDEquals(Class< ? > clazz, Object proxy, Object other) throws Throwable {
    if (other == null) {
      return false;
    }
    if (!(clazz.isAssignableFrom(other.getClass()))) {
      return false;
    }
    Method getUUID = clazz.getMethod("getUUID");
    Object myUUID = mockObject.invoke(new Invocation(proxy,getUUID));
    Object otherUUID = getUUID.invoke(other);
    return otherUUID.equals(myUUID);  
  }
  
  public static boolean isAnImpostClass(Class< ? >[] classes, Class< ? > clazz) {
    for (Class< ? > c : classes) {
      if (clazz.isAssignableFrom(c)) {
        return true;
      }
    }
    return false;
  }
  
  public void setClassesToImpost(Class< ? >[] classes) {
    this.classes = classes;
  }

  public void setMockObject(Invokable mock) {
    this.mockObject = mock;
  }
  public void setType(Class< ? > clazz) {
    this.clazz = clazz;
  }
}
