/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.Set;

import junit.framework.TestCase;

import org.ow2.bonita.facade.WorkflowElementFactory;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.IdFactory;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;

/**
 * @author Pierre Vigneras
 * @date 16 mai 08
 */
public class ProcessFullInstImplTest extends TestCase {

  @SuppressWarnings("unchecked")
  public void testCopy() {
    final ProcessInstanceUUID instanceUUID = IdFactory.getNewInstanceUUID();
    final PackageDefinitionUUID packageUUID = IdFactory.getNewPackageUUID();
    final ProcessDefinitionUUID processUUID = IdFactory.getNewProcessUUID();
    final ActivityInstanceUUID activity1UUID = IdFactory.getNewActivityUUID();
    final String activity1Id = IdFactory.getNewActivityId();
    final String iterationId = IdFactory.getNewIterationId();
    final String activityInstanceId = IdFactory.getNewActivityInstanceId();
    final ActivityFullInstance<TaskInstance> activity1 =
      WorkflowElementFactory.getNewActivity(activity1UUID, activity1Id, packageUUID, processUUID,
        instanceUUID, iterationId, activityInstanceId);
    final ActivityInstanceUUID activity2UUID = IdFactory.getNewActivityUUID();
    final String activity2Id = IdFactory.getNewActivityId();
    final ActivityFullInstance<TaskInstance> activity2 =
      WorkflowElementFactory.getNewActivity(activity2UUID, activity2Id, packageUUID, processUUID,
        instanceUUID, iterationId, activityInstanceId);
    final ProcessFullInstance instance =
      WorkflowElementFactory.getNewInstance(instanceUUID,
          packageUUID, processUUID, activity1, activity2);
    assertNotNull(instance.getActivities());
    assertTrue(instance.getActivities().size() == 2);
    assertNotNull(instance.getActivity(activity1Id, iterationId, activityInstanceId));
    assertEquals(instance.getActivity(activity1Id, iterationId, activityInstanceId).getActivityId(), activity1Id);
    assertNotNull(instance.getActivity(activity2Id, iterationId, activityInstanceId));
    assertEquals(instance.getActivity(activity2Id, iterationId, activityInstanceId).getActivityId(), activity2Id);

    final ProcessFullInstance copy = new ProcessFullInstanceImpl(instance);
    assertEquals(instanceUUID, copy.getUUID());
    assertEquals(packageUUID, copy.getPackageDefinitionUUID());
    assertEquals(processUUID, copy.getProcessDefinitionUUID());
    final Set<ActivityFullInstance<ActivityBody>> instanceActivities = instance.getActivities();
    final Set<ActivityFullInstance<ActivityBody>> copyActivities = copy.getActivities();
    assertNotSame(instanceActivities, copyActivities);
    assertTrue(instanceActivities.size() == copyActivities.size());
    assertNotNull(copy.getActivity(activity1Id, iterationId, activityInstanceId));
    assertNotNull(copy.getActivity(activity2Id, iterationId, activityInstanceId));
  }

}
