/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.integration.hook;

import junit.framework.Assert;

import org.ow2.bonita.definition.Hook;
import org.ow2.bonita.facade.QueryAPIAccessor;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.var.Enumeration;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;


/**
 * @author Pierre Vigneras
 * @date 5 mai 08
 */
public abstract class AbstractHook implements Hook {

  protected abstract void checkState(ActivityInstance<ActivityBody> activityInstance);

  public void execute(QueryAPIAccessor accessor, ActivityInstance<ActivityBody> activityInstance) throws Exception {
    Assert.assertNotNull("ActivityInstance is null !", activityInstance);

    QueryRuntimeAPI queryRuntimeAPI = accessor.getQueryRuntimeAPI();
    ActivityInstanceUUID activityUUID = activityInstance.getUUID();
    ProcessInstanceUUID instanceUUID = activityInstance.getProcessInstanceUUID();

    checkState(activityInstance);

    // INIT state so there is no record.
    try {
      queryRuntimeAPI.getActivityInstances(instanceUUID, "secondActivity");
      Assert.fail("this activity does not exist yet in records");
    } catch (ActivityNotFoundException e) {
      // normal execution
    }

    //check if the process variable is accessible and correctly initialized
    Enumeration processVarValue = (Enumeration) queryRuntimeAPI.getProcessInstanceVariable(instanceUUID, "processVar");
    Assert.assertNotNull("processVar is null !", processVarValue);
    Assert.assertNotNull("processVar has no selected value !", processVarValue.getSelectedValue());
    Assert.assertEquals("1", processVarValue.getSelectedValue());

    //check if the firstActivity (current) var is accessible and correctly initialized
    String firstActivityVarValue =
      (String) queryRuntimeAPI.getActivityInstanceVariable(activityUUID, "firstActivityVar");
    Assert.assertNotNull("firstActivityVar is null !", firstActivityVarValue);
    Assert.assertEquals("initial1", firstActivityVarValue);

    //check that the secondActivity var is not yet accessible
    try {
      queryRuntimeAPI.getActivityInstanceVariable(activityUUID, "secondActivityVar");
      Assert.fail("secondActivityVar is not null !");
    } catch (VariableNotFoundException e) {
      // OK
      Assert.assertEquals(activityUUID, e.getActivityUUID());
      Assert.assertEquals("secondActivityVar", e.getVariableId());
    }
  }
}
