/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.integration.routeSplitJoin;

import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.var.Enumeration;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.hook.DefaultTestHook;
import org.ow2.bonita.integration.task.AdminRoleMapper;
import org.ow2.bonita.util.BonitaException;

/**
 * Testing process starting with Split (either Automatic, task or route)
 */
public class SplitStartingNodeTest extends APITestCase {

  public void testRejectWithAuto() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitAutomaticStartingNode.xpdl");
    String processId = "splitAutomaticStartingNode";
    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class, MailAccept.class, MailReject.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();
    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    // check mailReject hook has been executed
    String statusMail = (String) getQueryRuntimeAPI().getProcessInstanceVariable(instanceUUID, "StatusMail");
    assertEquals("mailReject sent", statusMail);
    checkExecutedOnce(instanceUUID, new String[]{"Approval", "Rejection", "BonitaEnd"});

    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testAcceptWithAuto() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitAutomaticStartingNode.xpdl");
    String processId = "splitAutomaticStartingNode";
    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class, MailAccept.class, MailReject.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    Map<String, Object> variables = new HashMap<String, Object>();
    Set<String> possibleValues = new HashSet<String>();
    possibleValues.add("grant");
    possibleValues.add("rejected");
    Enumeration enumeration = new Enumeration(possibleValues, "grant");
    variables.put("approval_decision", enumeration);

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID, variables);
    checkExecutedOnce(instanceUUID, new String[]{"Approval", "Acceptance", "BonitaEnd"});

    // check mailAccept hook has been executed
    String statusMail = (String) getQueryRuntimeAPI().getProcessInstanceVariable(instanceUUID, "StatusMail");
    assertEquals("mailAccept sent", statusMail);

    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testRejectWithTask() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitTaskStartingNode.xpdl");
    String processId = "splitTaskStartingNode";
    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class, MailAccept.class, MailReject.class, AdminRoleMapper.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();
    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    checkStopped(instanceUUID, new String[]{});
    executeTask(instanceUUID, "Approval");
    checkExecutedOnce(instanceUUID, new String[]{"Approval", "Rejection", "BonitaEnd"});

    // check mailReject hook has been executed
    String statusMail = (String) getQueryRuntimeAPI().getProcessInstanceVariable(instanceUUID, "StatusMail");
    assertEquals("mailReject sent", statusMail);

    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testAcceptWithTask() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitTaskStartingNode.xpdl");
    String processId = "splitTaskStartingNode";
    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class, MailAccept.class, MailReject.class, AdminRoleMapper.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    Map<String, Object> variables = new HashMap<String, Object>();
    Set<String> possibleValues = new HashSet<String>();
    possibleValues.add("grant");
    possibleValues.add("rejected");
    Enumeration enumeration = new Enumeration(possibleValues, "grant");
    variables.put("approval_decision", enumeration);

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID, variables);

    checkStopped(instanceUUID, new String[]{});

    executeTask(instanceUUID, "Approval");

    checkExecutedOnce(instanceUUID, new String[]{"Approval", "Acceptance", "BonitaEnd"});

    String statusMail = (String) getQueryRuntimeAPI().getProcessInstanceVariable(instanceUUID, "StatusMail");
    assertEquals("mailAccept sent", statusMail);

    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testRejectWithRoute() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitRouteStartingNode.xpdl");
    String processId = "splitRouteStartingNode";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class, MailAccept.class, MailReject.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();
    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);

    checkExecutedOnce(instanceUUID, new String[]{"Approval", "Rejection", "BonitaEnd"});

    // check mailReject hook has been executed
    String statusMail = (String) getQueryRuntimeAPI().getProcessInstanceVariable(instanceUUID, "StatusMail");

    assertEquals("mailReject sent", statusMail);

    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testAcceptWithRoute() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitRouteStartingNode.xpdl");
    String processId = "splitRouteStartingNode";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class, MailAccept.class, MailReject.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    Map<String, Object> variables = new HashMap<String, Object>();
    Set<String> possibleValues = new HashSet<String>();
    possibleValues.add("grant");
    possibleValues.add("rejected");
    Enumeration enumeration = new Enumeration(possibleValues, "grant");
    variables.put("approval_decision", enumeration);

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID, variables);
    checkExecutedOnce(instanceUUID, new String[]{"Approval", "Acceptance", "BonitaEnd"});

    String statusMail = (String) getQueryRuntimeAPI().getProcessInstanceVariable(instanceUUID, "StatusMail");
    assertEquals("mailAccept sent", statusMail);

    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

  public void testProcessStartingWithRouteNoSplit() throws BonitaException {
    URL xpdlUrl = this.getClass().getResource("splitRoute1TrStartingNode.xpdl");
    String processId = "splitRoute1TrStartingNode";

    ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl,
        DefaultTestHook.class)).get(processId);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    ProcessDefinitionUUID processUUID = process.getUUID();

    ProcessInstanceUUID instanceUUID = getRuntimeAPI().instantiateProcess(processUUID);
    checkExecutedOnce(instanceUUID, new String[]{"r1", "a", "BonitaEnd"});
    getRuntimeAPI().deleteProcessInstance(instanceUUID);
    getManagementAPI().undeploy(packageUUID);
    getManagementAPI().deletePackage(packageUUID);
  }

}
