/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.impl;

import java.net.URL;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.env.generator.DbHistoryEnvGenerator;
import org.ow2.bonita.env.generator.EnvGenerator;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.PackageNotFoundException;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.services.handlers.impl.CleanJournalUndeployedPackageHandler;
import org.ow2.bonita.util.BonitaException;


public class CleanJournalUndeployedPackageHandlerTest extends
    APITestCase {

  @SuppressWarnings("unchecked")
  public static EnvGenerator getEnvGenerator() {
    final EnvGenerator envGenerator = new DbHistoryEnvGenerator();
    envGenerator.setUndeployedPackageHandlerType(CleanJournalUndeployedPackageHandler.class);
    return envGenerator;
  }

  public void testCleanJournalUndeployedPackageHandler() throws BonitaException {
    final URL xpdlUrl = this.getClass().getResource("simple_process.xpdl");
    final String processId = "simple";

    final ProcessDefinition process = getManagementAPI().deployBar(getBarContent(xpdlUrl)).get(processId);
    final PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    final ProcessDefinitionUUID processUUID = process.getUUID();

    // Check there is an package record in the journal
    assertNotNull(getQueryDefinitionAPI().getPackage(packageUUID));
    // Check there is an process record in the journal
    assertNotNull(getQueryDefinitionAPI().getProcess(processUUID));

    // undeploy package
    getManagementAPI().undeploy(packageUUID);

    // check there is no record for this package in the journal or in the archive.
    try {
      getQueryDefinitionAPI().getPackage(packageUUID);
      fail("This package does not exist");
    } catch (final PackageNotFoundException e) {
      assertEquals(packageUUID, e.getPackageUUID());
    }

  }
}


