/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.record;

import java.util.Date;
import java.util.Set;

import org.ow2.bonita.env.GlobalEnvironmentFactory;
import org.ow2.bonita.env.generator.DbHistoryEnvGenerator;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.TaskFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.impl.TaskFullInstanceImpl;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.persistence.QuerierDbSession;
import org.ow2.bonita.persistence.db.QuerierDbSessionImpl;
import org.ow2.bonita.pvm.env.Environment;
import org.ow2.bonita.pvm.internal.cmd.Command;
import org.ow2.bonita.pvm.internal.cmd.CommandService;
import org.ow2.bonita.util.EnvConstants;
import org.ow2.bonita.util.EnvTool;


/**
 * @author Guillaume Porcher
 *
 */
public class DbTaskInstTest extends AbstractTaskInstTest {

  protected CommandService commandService;

  @Override
  protected void setUp() throws Exception {
    super.setUp();
    GlobalEnvironmentFactory.setEnvironmentFactory(new DbHistoryEnvGenerator().createEnvironmentFactory());
    this.commandService = GlobalEnvironmentFactory.getEnvironmentFactory().get(CommandService.class);
  }

  @Override
  protected void tearDown() throws Exception {
    GlobalEnvironmentFactory.getEnvironmentFactory().close();
    GlobalEnvironmentFactory.setEnvironmentFactory(null);
    this.commandService = null;
    super.tearDown();
  }

  private static QuerierDbSession getQuerierDbSession() {
    return EnvTool.getQuerierDbSession(EnvConstants.CORE_QUERIER_SESSION_NAME);
  }
  private static class SaveTask implements Command<TaskFullInstanceImpl> {
    /**
     * 
     */
    private static final long serialVersionUID = -183655847991256832L;
    private final TaskFullInstanceImpl task;

    public SaveTask(final TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(final Environment environment) throws Exception {
      getQuerierDbSession().save(this.task);
      return this.task;
    }
  }

  private static class UpdateTask implements Command<TaskFullInstanceImpl> {
    /**
     * 
     */
    private static final long serialVersionUID = 1170354948937168716L;
    private TaskFullInstanceImpl task;

    public UpdateTask(final TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(final Environment environment) throws Exception {
      this.task = getQuerierDbSession().get(TaskFullInstanceImpl.class, this.task.getDbid());
      if (this.task != null) {
        assertNotNull(this.task.toString());
      }
      return this.task;
    }
  }

  class MergeTask implements Command<TaskFullInstanceImpl> {
    /**
     * 
     */
    private static final long serialVersionUID = -6323609392731073032L;
    private final TaskFullInstanceImpl task;

    public MergeTask(final TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(final Environment environment) throws Exception {
      ((QuerierDbSessionImpl)getQuerierDbSession()).getSession().merge(this.task);
      return this.task;
    }
  }

  private static class RemoveTask implements Command<TaskFullInstanceImpl> {
    /**
     * 
     */
    private static final long serialVersionUID = -4325384512700228634L;
    private final TaskFullInstanceImpl task;

    public RemoveTask(final TaskFullInstanceImpl task) {
      this.task = task;
    }

    public TaskFullInstanceImpl execute(final Environment environment) throws Exception {
      environment.get(QuerierDbSession.class).delete(this.task);
      return this.task;
    }
  }

  @Override
  protected TaskInstance createTaskRecord(final TaskUUID taskUUID, final String userId, final Set<String> candidates,
      final ActivityState state, final Date createdDate) {

    final TaskFullInstanceImpl taskRecord = new TaskFullInstanceImpl(taskUUID,
        state,
        createdDate,
        "user");
    taskRecord.setTaskAssign(state, null, candidates, userId);

    this.commandService.execute(new SaveTask(taskRecord));

    return this.commandService.execute(new UpdateTask(taskRecord));
  }

  private TaskInstance update(final TaskInstance taskInstance) {
    this.commandService.execute(new MergeTask((TaskFullInstanceImpl) taskInstance));
    return this.commandService.execute(new UpdateTask((TaskFullInstanceImpl) taskInstance));
  }

  @Override
  protected void remove(final TaskInstance taskInstance) {
    this.commandService.execute(new RemoveTask((TaskFullInstanceImpl) taskInstance));
  }

  @Override
  protected TaskInstance setEndedDate(final TaskInstance taskInstance, final Date endedTime) {
    ((TaskFullInstance)taskInstance).setEndedDate(endedTime);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setStartedBy(final TaskInstance taskInstance, final String startActor) {
    ((TaskFullInstance)taskInstance).setStartedBy(startActor);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setEndedBy(final TaskInstance taskInstance, final String endActor) {
    ((TaskFullInstance)taskInstance).setEndedBy(endActor);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setStartedDate(final TaskInstance taskInstance, final Date startedTime) {
    ((TaskFullInstance)taskInstance).setStartedDate(startedTime);
    return update(taskInstance);
  }

  @Override
  protected TaskInstance setReadyDate(final TaskInstance taskInstance, final Date readyTime) {
    ((TaskFullInstance)taskInstance).setReadyDate(readyTime);
    return update(taskInstance);
  }

}
