package org.ow2.bonita.textui;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.security.auth.login.LoginException;

import org.ow2.bonita.APITestCase;
import org.ow2.bonita.example.aw.hook.Accept;
import org.ow2.bonita.example.aw.hook.Reject;
import org.ow2.bonita.example.aw.mapper.ApprovalMapper;
import org.ow2.bonita.facade.exception.DeploymentException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BusinessArchiveFactory;
import org.ow2.bonita.util.Misc;


public class ConsoleTest extends APITestCase {

  private static final String LOGINCONFIG = "Bonita";
  private static final String USERNAME = "john";
  private static final String PASSWORD = "bpm";
  private static final String PACKAGE_ID = "ApprovalWorkflow";
  private static final String PROCESS_ID = "ApprovalWorkflow";
  private static final String XPDLFILE = "ApprovalWorkflow.xpdl";
  @Override
  protected String getLogin() {
    return USERNAME;
  }

  @Override
  protected String getPassword() {
    return PASSWORD;
  }


  public void testDeployNullBar() throws BonitaException, IOException {
    try {
      Console.deploy(null);
      fail("Bar file must be not null");
    } catch (final IllegalArgumentException e) {
      // ok
    }
  }


  public void testUndeployNullProcessId() throws BonitaException {
    try {
      Console.undeploy(null, null);
      fail("PackageId and ProcessId must be not null");
    } catch (final IllegalArgumentException e) {
      // ok
    }
  }


  public void testMainNullArgs() throws BonitaException, IOException, LoginException {
    final String[] arguments = {null, null, null};
    try {
      Console.main(arguments);
      fail(Console.USAGE);
    } catch (final IllegalArgumentException e) {
      // ok
    }
  }

  @SuppressWarnings("unchecked")
  public void testDeployTwice() throws BonitaException, IOException, LoginException {
    final Class[] classes = {
      Accept.class, Reject.class, ApprovalMapper.class
    };
    final URL url = ConsoleTest.class.getResource(XPDLFILE);
    final byte[] xpdlContent = Misc.getAllContentFrom(url);
    final byte[] bar = BusinessArchiveFactory.getBusinessArchiveContent(XPDLFILE, xpdlContent, classes);
    try {
      Console.deploy(bar);
      Console.deploy(bar);
      fail("Repository already contains this package");
    } catch (final DeploymentException e) {
      Console.undeploy(PACKAGE_ID, PROCESS_ID);
    }
  }

  @SuppressWarnings("unchecked")
  public void testDeployUndeployGoodArgsStandard() throws BonitaException, IOException {
    final Class[] classes = {Accept.class, Reject.class, ApprovalMapper.class};
    final URL url = ConsoleTest.class.getResource(XPDLFILE);
    final byte[] xpdlContent = Misc.getAllContentFrom(url);
    final byte[] bar = BusinessArchiveFactory.getBusinessArchiveContent(XPDLFILE, xpdlContent, classes);

    Console.deploy(bar);

    Console.undeploy(PACKAGE_ID, PROCESS_ID);
  }

  public void testUndeployInexistantProcessId() throws BonitaException {
    try {
      Console.undeploy(PACKAGE_ID, PROCESS_ID);
      fail("can't find a process with Id " + PROCESS_ID + " in package with Id " + PACKAGE_ID);
    } catch (final ProcessNotFoundException e) {
      // ok
    }
  }

  public void testMainBadAction() throws Exception {
    tearDown();
    try {
      final File file = Misc.createTempFile(ConsoleTest.class.getName(), ".bar", null);
      file.deleteOnExit();
      final String fileName = file.getAbsolutePath();
      final String[] argumentsForDeploy = {
        "ACTION", fileName, LOGINCONFIG, USERNAME, PASSWORD
      };
      Console.main(argumentsForDeploy);
      final String[] argumentsForUndeploy = {
        Console.UNDEPLOY, PACKAGE_ID, PACKAGE_ID, PROCESS_ID, LOGINCONFIG, USERNAME, PASSWORD
      };
      Console.main(argumentsForUndeploy);
      fail(Console.ACTION_USAGE);
    } catch (final IllegalArgumentException e) {
      //ok
    }

  }

  @SuppressWarnings("unchecked")
  public void testMainGoodArgs() throws Exception {
    final Class[] classes = {
      Accept.class, Reject.class, ApprovalMapper.class
    };
    final URL url = ConsoleTest.class.getResource(XPDLFILE);
    final byte[] xpdlContent = Misc.getAllContentFrom(url);
    tearDown();
    final File file = Misc.createTempFile(ConsoleTest.class.getName(), ".bar", null);
    file.deleteOnExit();
    byte[] barContent = BusinessArchiveFactory.getBusinessArchiveContent(XPDLFILE, xpdlContent, classes);
    BusinessArchiveFactory.generateBusinessArchiveFile(file, barContent);
    final String[] argumentsForDeploy = {
      Console.DEPLOY, file.getAbsolutePath(), LOGINCONFIG, USERNAME, PASSWORD
    };
    Console.main(argumentsForDeploy);
    final String[] argumentsForUndeploy = {
      Console.UNDEPLOY, PACKAGE_ID, PROCESS_ID, LOGINCONFIG, USERNAME, PASSWORD
    };
    Console.main(argumentsForUndeploy);
  }

}
